#! /usr/bin/env python
# -*- coding: utf-8 -*-

def updateHistoryData(whiteAids=[], blackVids=[]):
	subjectDict, worksDict = createWorksDict(aids, blackVids)

def getWorksActionInfo(vid):
	worksInfo = {}



# return
# subjectDict: aid subject(count:作品数量)
# worksDict: vid works(duration:时长 title:标题 aid:认证号id publishTime:发布时间 score:分数)
def createWorksDict(whiteAids=[], blackVids=[]):
	subjectDict = {}
	worksDict = {}
	for hit in es.scrollSearch('works_video_read', 'video', {"size":1000}):
		worksInfo = hit['_source']
		if not worksInfo.has_key('vid') or not worksInfo.has_key('aid') or (worksInfo.has_key('state') and worksInfo['state'] == 1) or (worksInfo.has_key('authorstate') and worksInfo['authorstate'] == 1):
			continue
		if not worksInfo.has_key('publishtime') or worksInfo['publishtime'] == None:
			continue
		publishTime = int(time.time() - es.esTimeToTime(worksInfo['publishtime'], True)) / 3600
		if publishTime > 10000:
			continue
		aid = worksInfo['aid']
		vid = worksInfo['vid']
		if (len(aids) != 0 and not aid in aids) or vid in blackVids:
			continue
		if subjectDict.has_key(aid):
			subject = subjectDict[aid]
			subject['count'] += 1
		else:
			subject = subjectDict[aid] = {'count': 1}
		duration = worksInfo['duration'] if worksInfo.has_key('duration') else 0
		durationIndex = 12
		for index, value in enumerate(DurationSections):
			if duration < value:
				durationIndex = index
				break
		worksDict[vid] = {'title': worksInfo['title'], 'aid': aid, 'publishTime': publishTime, 'playCount': worksInfo['playcount'] if worksInfo.has_key('playcount') else 0, 'playTime': 0, 'timeCount': 0, 'show': 0, 'showPlay': 0, 'recShow': 0, 'recShowPlay': 0, 'durationIndex': durationIndex, 'durationAvgPlay': 0, 'statis': [0, 0, 0, 0, 0, 0, 0, 0, 0, duration, size]}

	return subjectDict, worksDict




#! /usr/bin/env python
# -*- coding: utf-8 -*-

import time, random, math
from .. import es
from bftWorks import BftWorks

def updateWorks():
	#endTime = int(time.time())
	lastCheckTime = 1547568000

	endTime = 1547568000
	vids = []
	for index in range(0, 80):
		vids.append(str(index))
	worksDict = getWorksDict(vids, endTime)

	while True:
		if random.randint(1, 2) == 1:
			vids.append(str(time.time()))
		worksDict = updateWorksDict(worksDict, vids, endTime)
		endTime = update(worksDict, endTime)
		
		if endTime - lastCheckTime >= 24 * 60 * 60:
			dailyCheck(worksDict, lastCheckTime)
			lastCheckTime = endTime

		saveWorksToEs(worksDict)

		saveRecord(worksDict, endTime)

		print(es.timeToEsLocalTime(endTime))
		time.sleep(30)

def update(worksDict, startTime):
	#endTime = int(time.time() - 90)
	endTime = startTime + 10 * 60
	for hit in es.searchStatisticsDataByBody("works_actions", {"query": {"bool": {"must": [{"range": {"time": {"gte": startTime * 1000, "lt": endTime * 1000}}}, {"terms": {"action": [2, 3, 5, 12, 13, 16]}}]}}, "size": 10000}):
		action = hit['_source']
		vid = action['vid']
		if random.randint(1, 5) == 1:
			for i in range(0, 100):
				vid = random.sample(worksDict.keys(), 1)[0]
				if worksDict[vid].pool in [BftWorks.Pools['FlowFirst'], BftWorks.Pools['FlowSecond'], BftWorks.Pools['FlowThird']]:
					break
		if worksDict.has_key(vid):
			worksDict[vid].updateAction(action)

	for works in worksDict.values():
		works.updateStatus()

	return endTime

def updateWorksDict(worksDict, vids, endTime):
	newWorksDict = {}
	for vid in vids:
		if worksDict.has_key(vid):
			newWorksDict[vid] = worksDict[vid]
		else:
			works = BftWorks(vid)
			works.updateHistoryActions(endTime)
			newWorksDict[vid] = works
	return newWorksDict

def getWorksDict(vids, endTime):
	worksDict = {}
	for vid in vids:
		worksDict[vid] = BftWorks(vid)
	syncEsWorks(worksDict)
	for works in worksDict.values():
		works.updateHistoryActions(endTime)
	return worksDict

def dailyCheck(worksDict, lastCheckTime):
	awaitWorks = [[], [], []]
	riseCounts = [0, 0, 0]

	for works in worksDict.values():
		if works.status == BftWorks.Status['Await']:
			awaitWorks[works.getPool()].append(works)
		if works.status != BftWorks.Status['Fail']:
			for record in works.records:
				if record['time'] > lastCheckTime:
					riseCounts[works.getPool()] += 1

	for index, worksList in enumerate(awaitWorks):
		worksList.sort(reverse=True, key=lambda item: item.score)

		riseCount = riseCounts[index]
		polishCount = (len(worksList) + riseCount) * 0.2 - riseCount

		for index, works in enumerate(worksList):
			if index < polishCount - 1:
				works.rise(2)
			elif works.isResurgence():
				works.fail()
			else:
				works.resurgence()

def saveRecord(worksDict, endTime):
	record = {'time': es.timeToEsLocalTime(endTime), 'flowFirstCount': 0, 'flowSecondCount': 0, 'flowThirdCount': 0, 'successCount': 0, 'failCount': 0}
	for works in worksDict.values():
		if works.status == BftWorks.Status['Normal']:
			record[{0: 'flowFirstCount', 1: 'flowSecondCount', 2: 'flowThirdCount'}[works.getPool()]] += 1
		elif works.status == BftWorks.Status['Success']:
			record['successCount'] += 1
		elif works.status in [BftWorks.Status['Await'], BftWorks.Status['Fail']]:
			record['failCount'] += 1
	return es.updateStatisticsData('works_filter_pool_record', {int(endTime): record})

def syncEsWorks(worksDict):
	for hit in es.searchStatisticsData('works_filter_pool'):
		vid = hit['_id']
		if worksDict.has_key(vid):
			worksDict[vid].syncEsData(hit['_source'])

def saveWorksToEs(worksDict):
	worksData = {}
	for vid, works in worksDict.items():
		worksData[vid] = works.toSaveEsData()
	es.updateStatisticsData('works_filter_pool', worksData)

	


import setuptools, shutil

try:
    shutil.rmtree('build')
    shutil.rmtree('dist')
    shutil.rmtree('iduoliao_ml.egg-info')
except Exception as error:
    print('Error: ' + str(error))

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="iduoliao_ml_dev",
    version="1.1.55",
    author="legend",
    author_email="huangzhen@gzecy.com",
    description="iduoliao machine learning",
    long_description=long_description,
    long_description_content_type="text/markdown",
    #url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    #packages=['iduoliao_ml', 'iduoliao_ml.data', 'iduoliao_ml.es', 'iduoliao_ml.ilog',  'iduoliao_ml.imatrix',  'iduoliao_ml.recommend'],
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)